#include "viewer.h"
#include "nuple.h"
#include <stdlib.h>
#include <stdio.h>
#include <iostream>

using namespace std;
using namespace qglviewer;

namespace
{
  void
  drawCube(const Vec& m,const Vec& M)
  {
    const Vec corners[8] =
      {
				Vec(m[0],m[1],m[2]),
				Vec(M[0],m[1],m[2]),
				Vec(m[0],M[1],m[2]),
				Vec(M[0],M[1],m[2]),
				Vec(m[0],m[1],M[2]),
				Vec(M[0],m[1],M[2]),
				Vec(m[0],M[1],M[2]),
				Vec(M[0],M[1],M[2])
      };
    static const unsigned short faceCorners[6][4] =	
      {
				{ 0,4,6,2 },
				{ 5,1,3,7 },
				{ 0,1,5,4 },
				{ 3,2,6,7 },
				{ 0,2,3,1 },
				{ 6,4,5,7 }
      };  
    glBegin(GL_QUADS);
    for (unsigned short f=0;f<6;++f)
		{
			const unsigned short* face = faceCorners[f];
			glVertex3fv(corners[face[0]]);
			glVertex3fv(corners[face[1]]);
			glVertex3fv(corners[face[2]]);
			glVertex3fv(corners[face[3]]);
		}
    glEnd();
  }
}
//************************************************************
// Implementation of Viewer
//************************************************************
Viewer::Viewer(QWidget* parent)
  : QGLViewer(parent),
    wireFrame(false),
    drawBBox(false),
    drawGrid(false),
    drawLOD(false)
{
}
void
Viewer::loadModel(const char* fileName)
{
  FILE* f = fopen(fileName,"r");
  if (!f)
	{
		cerr<<"Cannot load "<<fileName<<endl;
		return;
	}
  // Read the number of primitives  
  unsigned int nbPrimitives;
  fread(&nbPrimitives,sizeof(unsigned int),1,f);
  for (unsigned int i=0;i<nbPrimitives;++i)
	{
		// Read the vertices
		unsigned int nbVertices;
		fread(&nbVertices,sizeof(unsigned int),1,f);
		for (unsigned int i=0;i<nbVertices;++i)
		{
			float c[3];
			fread(c,sizeof(float),3,f);
			vertices.push_back(Vec(c[0],c[1],c[2]));
			bbox += c;
		}
		// Read the faces
		unsigned int nbFaces;
		fread(&nbFaces,sizeof(unsigned int),1,f);
		for (unsigned int i=0;i<nbFaces;++i)
		{
			unsigned int v[3];
			fread(v,sizeof(unsigned int),3,f);	
			faces.push_back(v[0]);
			faces.push_back(v[1]);
			faces.push_back(v[2]);
			Vec a = vertices[v[0]];
			Vec b = vertices[v[1]];
			Vec c = vertices[v[2]];
			faceNormals.push_back(((b-a)^(c-a)).unit());
		}
	}
  fclose(f);
  cout<<"number of faces = "<<faces.size()<<endl;
  setSceneCenter(bbox.center());
  setSceneRadius(bbox.sizeMax()/2.0f);
  showEntireScene();
}
int 
Viewer::idCellContaining(const Vec& v) const
{
  const Vec& o = bbox.min();
  int i = static_cast<int>((v.x-o.x)/size);
  int j = static_cast<int>((v.y-o.y)/size);
  int k = static_cast<int>((v.z-o.z)/size);
  return (k*nbJ+j)*nbI+i;
}
void
Viewer::computeLOD(float percentage)
{
  //////////////////////////////////////////////////////////////
  // Build a grid from the bbox. Each cell is a cube of *fixed* size
  // so the number of cell is not the same in every direction.
  size = percentage*bbox.sizeMax();


  // To do:
  // Section 2.2 (a,b): Put here the code to allocate the grid of cells.

  //////////////////////////////////////////////////////////////
  // Run thru all vertices of the model and find in which cell they
  // are. Store in the cell the list of vertices that are contained.

  // To do:
  // Section 2.3 (a): Put here the code to cluster the vertices

  //////////////////////////////////////////////////////////////
  // Now run thru each cell and compute the associated representant from
  // the list of points stored in the cell.

  // To do:
  // Section 2.3 (b): Put here the code to compute representants


  //////////////////////////////////////////////////////////////
  // Finally we run thru all faces and build triangles

  // To do:
  // Section 2.4 (a): Put here the code to build simplified triangles

  cout<<"number of LOD triangles = "<<lodTriangleNormals.size()<<endl;
}
void
Viewer::draw()
{
  glPushAttrib(GL_ALL_ATTRIB_BITS);
  glColor3f(1.0f,1.0f,1.0f);
  // To do:
  // Section 2.1 (a,b): Put here the code to display the model

  // To do:
  // Section 2.2 (b): Put here the code to display the grid

  // To do:
  // Section 2.4 (b): Put here the code to display simplification
}
void
Viewer::keyPressEvent(QKeyEvent* e)
{
  switch (e->key())
    {
    case Key_D:
      qWarning("Key_D pressed!");
      updateGL();
      break;
    default:
      QGLViewer::keyPressEvent(e);
    }
}
